@@ -12,6 +12,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import json
+import zlib
+
 from glanceclient.v2 import client as glanceclient
 from heatclient.v1 import client as heatclient
 import ironic_inspector_client
@@ -22,6 +25,8 @@ from mistral.utils.openstack import keystone as keystone_utils
 from mistralclient.api import client as mistral_client
 from novaclient.client import Client as nova_client
 from swiftclient import client as swift_client
+from swiftclient import exceptions as swiftexceptions
+from tripleo_common import constants
 
 
 class TripleOAction(base.Action):
@@ -119,8 +124,81 @@ class TripleOAction(base.Action):
         keystone_endpoint = keystone_utils.get_endpoint_for_project('keystone')
         nova_endpoint = keystone_utils.get_endpoint_for_project('nova')
 
-        nc = nova_client(2, username=ctx.user_name, auth_token=ctx.auth_token,
-                         auth_url=keystone_endpoint.url, url=nova_endpoint.url,
-                         project_id=ctx.project_id)
-        nc.client.management_url = nova_endpoint.url
-        return nc
+        client = nova_client(
+            2,
+            username=None,
+            api_key=None,
+            service_type='compute',
+            auth_token=ctx.auth_token,
+            tenant_id=ctx.project_id,
+            region_name=keystone_endpoint.region,
+            auth_url=keystone_endpoint.url,
+            insecure=ctx.insecure
+        )
+
+        client.client.management_url = keystone_utils.format_url(
+            nova_endpoint.url,
+            {'tenant_id': ctx.project_id}
+        )
+
+        return client
+
+    def _cache_key(self, plan_name, key_name):
+        return "__cache_{}_{}".format(plan_name, key_name)
+
+    def cache_get(self, plan_name, key):
+        """Retrieves the stored objects
+
+        Returns None if there are any issues or no objects found
+
+        """
+
+        swift_client = self.get_object_client()
+        try:
+            headers, body = swift_client.get_object(
+                constants.TRIPLEO_CACHE_CONTAINER,
+                self._cache_key(plan_name, key)
+            )
+            result = json.loads(zlib.decompress(body).decode())
+            return result
+        except swiftexceptions.ClientException:
+            # cache does not exist, ignore
+            pass
+        except ValueError:
+            # the stored json is invalid. Deleting
+            self.cache_delete(plan_name, key)
+        return
+
+    def cache_set(self, plan_name, key, contents):
+        """Stores an object
+
+        Allows the storage of jsonable objects except for None
+        Storing None equals to a cache delete.
+
+        """
+
+        swift_client = self.get_object_client()
+        if contents is None:
+            self.cache_delete(plan_name, key)
+            return
+
+        try:
+            swift_client.head_container(constants.TRIPLEO_CACHE_CONTAINER)
+        except swiftexceptions.ClientException:
+            swift_client.put_container(constants.TRIPLEO_CACHE_CONTAINER)
+
+        swift_client.put_object(
+            constants.TRIPLEO_CACHE_CONTAINER,
+            self._cache_key(plan_name, key),
+            zlib.compress(json.dumps(contents).encode()))
+
+    def cache_delete(self, plan_name, key):
+        swift_client = self.get_object_client()
+        try:
+            swift_client.delete_object(
+                constants.TRIPLEO_CACHE_CONTAINER,
+                self._cache_key(plan_name, key)
+            )
+        except swiftexceptions.ClientException:
+            # cache or container does not exist. Ignore
+            pass
