@@ -41,6 +41,19 @@ class ClearBreakpointsAction(base.TripleOAction):
         update_manager.clear_breakpoints(self.refs)
 
 
+class CancelStackUpdateAction(base.TripleOAction):
+    def __init__(self, stack_id):
+        super(CancelStackUpdateAction, self).__init__()
+        self.stack_id = stack_id
+
+    def run(self):
+        heat = self.get_orchestration_client()
+        nova = self.get_compute_client()
+        update_manager = PackageUpdateManager(
+            heat, nova, self.stack_id, stack_fields={})
+        update_manager.cancel()
+
+
 class UpdateStackAction(templates.ProcessTemplatesAction):
 
     def __init__(self, timeout, container=constants.DEFAULT_CONTAINER_NAME):
@@ -55,7 +68,7 @@ class UpdateStackAction(templates.ProcessTemplatesAction):
         except heat_exc.HTTPNotFound:
             msg = "Error retrieving stack: %s" % self.container
             LOG.exception(msg)
-            return mistral_workflow_utils.Result("", msg)
+            return mistral_workflow_utils.Result(error=msg)
 
         parameters = dict()
         timestamp = int(time.time())
@@ -69,7 +82,7 @@ class UpdateStackAction(templates.ProcessTemplatesAction):
         except Exception:
             msg = "Error retrieving mistral environment: %s" % self.container
             LOG.exception(msg)
-            return mistral_workflow_utils.Result("", msg)
+            return mistral_workflow_utils.Result(error=msg)
 
         if 'parameter_defaults' not in wf_env.variables:
             wf_env.variables['parameter_defaults'] = {}
