@@ -78,14 +78,15 @@ class J2SwiftLoader(jinja2.BaseLoader):
 
 class UploadTemplatesAction(base.TripleOAction):
     """Upload default heat templates for TripleO."""
-    def __init__(self, container=constants.DEFAULT_CONTAINER_NAME):
+    def __init__(self, container=constants.DEFAULT_CONTAINER_NAME,
+                 templates_path=constants.DEFAULT_TEMPLATES_PATH):
         super(UploadTemplatesAction, self).__init__()
         self.container = container
+        self.templates_path = templates_path
 
     def run(self):
-        tht_base_path = constants.DEFAULT_TEMPLATES_PATH
         with tf.NamedTemporaryFile() as tmp_tarball:
-            tarball.create_tarball(tht_base_path, tmp_tarball.name)
+            tarball.create_tarball(self.templates_path, tmp_tarball.name)
             tarball.tarball_extract_to_swift_container(
                 self.get_object_client(),
                 tmp_tarball.name,
@@ -124,6 +125,7 @@ class ProcessTemplatesAction(base.TripleOAction):
         try:
             # write the template back to the plan container
             LOG.info("Writing rendered template %s" % yaml_f)
+            self.cache_delete(self.container, "tripleo.parameters.get")
             swift.put_object(
                 self.container, yaml_f, r_template)
         except swiftexceptions.ClientException as ex:
@@ -161,6 +163,16 @@ class ProcessTemplatesAction(base.TripleOAction):
                      % constants.OVERCLOUD_J2_ROLES_NAME)
             return
 
+        try:
+            j2_network_file = swift.get_object(
+                self.container, constants.OVERCLOUD_J2_NETWORKS_NAME)[1]
+            network_data = yaml.safe_load(j2_network_file)
+        except swiftexceptions.ClientException:
+            # Until t-h-t contains network_data.yaml we tolerate a missing file
+            LOG.warning("No %s file found, ignoring"
+                        % constants.OVERCLOUD_J2_ROLES_NAME)
+            network_data = []
+
         j2_excl_data = self._get_j2_excludes_file()
 
         try:
@@ -211,7 +223,7 @@ class ProcessTemplatesAction(base.TripleOAction):
             elif f.endswith('.j2.yaml'):
                 LOG.info("jinja2 rendering %s" % f)
                 j2_template = swift.get_object(self.container, f)[1]
-                j2_data = {'roles': role_data}
+                j2_data = {'roles': role_data, 'networks': network_data}
                 out_f = f.replace('.j2.yaml', '.yaml')
                 self._j2_render_and_put(j2_template, j2_data, out_f)
 
