@@ -23,6 +23,9 @@ OVERCLOUD_J2_NAME = "overcloud.j2.yaml"
 #: The name of custom roles data file used when rendering the jinja template.
 OVERCLOUD_J2_ROLES_NAME = "roles_data.yaml"
 
+#: The name of custom roles network data file used when rendering j2 templates.
+OVERCLOUD_J2_NETWORKS_NAME = "network_data.yaml"
+
 #: The name of custom roles excl file used when rendering the jinja template.
 OVERCLOUD_J2_EXCLUDES = "j2_excludes.yaml"
 
@@ -105,3 +108,14 @@ PLAN_NAME_PATTERN = '^[a-zA-Z0-9-]+$'
 # The default version of the Bare metal API to set in overcloudrc.
 # 1.29 is the latest API version in Ironic Ocata supported by ironicclient.
 DEFAULT_BAREMETAL_API_VERSION = '1.29'
+
+# The name of the file which holds the Mistral environment contents for plan
+# import/export
+PLAN_ENVIRONMENT = 'plan-environment.yaml'
+
+DEFAULT_DEPLOY_KERNEL_NAME = 'bm-deploy-kernel'
+
+DEFAULT_DEPLOY_RAMDISK_NAME = 'bm-deploy-ramdisk'
+
+# The name for the swift container to host the cache for tripleo
+TRIPLEO_CACHE_CONTAINER = "__cache__"
