@@ -12,9 +12,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-
-import six
-
 import json
 import os
 import yaml
@@ -27,7 +24,11 @@ from tripleo_common.image.exception import ImageSpecificationException
 class BaseImageManager(object):
     logger = log.getLogger(__name__ + '.BaseImageManager')
     APPEND_ATTRIBUTES = ['elements', 'options', 'packages']
-    CONFIG_SECTIONS = ['disk_images', 'uploads']
+    CONFIG_SECTIONS = (
+        DISK_IMAGES, UPLOADS, CONTAINER_IMAGES
+    ) = (
+        'disk_images', 'uploads', 'container_images'
+    )
 
     def __init__(self, config_files, images=None):
         self.config_files = config_files
@@ -48,7 +49,7 @@ class BaseImageManager(object):
         for config_file in self.config_files:
             if os.path.isfile(config_file):
                 with open(config_file) as cf:
-                    data = yaml.load(cf.read()).get(section)
+                    data = yaml.safe_load(cf.read()).get(section)
                     if not data:
                         return None
                     self.logger.debug('%s JSON: %s' % (section, str(data)))
@@ -74,7 +75,7 @@ class BaseImageManager(object):
                                                       attr)
 
                     # If a new key is introduced, add it.
-                    for key, value in six.iteritems(item):
+                    for key, value in item.items():
                         if key not in existing_image:
                             existing_image[key] = item[key]
 
@@ -86,5 +87,5 @@ class BaseImageManager(object):
 
     def json_output(self):
         self.logger.info('Using config files: %s' % self.config_files)
-        disk_images = self.load_config_files(self.CONFIG_SECTIONS[0])
+        disk_images = self.load_config_files(self.DISK_IMAGES)
         print(json.dumps(disk_images))
