@@ -21,9 +21,7 @@ from tripleo_common.actions import heat_capabilities
 from tripleo_common.tests import base
 
 
-MAPPING_YAML_CONTENTS = """root_template: /path/to/overcloud.yaml
-root_environment: /path/to/environment.yaml
-topics:
+MAPPING_YAML_CONTENTS = """topics:
   - title: Fake Single Environment Group Configuration
     description:
     environment_groups:
@@ -238,9 +236,11 @@ class UpdateCapabilitiesActionTest(base.TestCase):
         super(UpdateCapabilitiesActionTest, self).setUp()
         self.container_name = 'test-container'
 
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'cache_delete')
     @mock.patch(
         'tripleo_common.actions.base.TripleOAction.get_workflow_client')
-    def test_run(self, get_workflow_client_mock):
+    def test_run(self, get_workflow_client_mock, mock_cache):
 
         # setup mistral
         mistral = mock.MagicMock()
@@ -269,6 +269,48 @@ class UpdateCapabilitiesActionTest(base.TestCase):
             ]},
             action.run())
 
+        mock_cache.assert_called_once_with(
+            self.container_name,
+            "tripleo.parameters.get"
+        )
+
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'cache_delete')
+    @mock.patch(
+        'tripleo_common.actions.base.TripleOAction.get_workflow_client')
+    def test_run_purge_missing(self, get_workflow_client_mock, mock_cache):
+
+        # setup mistral
+        mistral = mock.MagicMock()
+        mocked_env = mock.MagicMock()
+        mocked_env.variables = {
+            'environments': [
+                {'path': '/path/to/overcloud-default-env.yaml'},
+                {'path': '/path/to/ceph-storage-env.yaml'},
+            ]
+        }
+        mistral.environments.get.return_value = mocked_env
+        get_workflow_client_mock.return_value = mistral
+
+        environments = {
+            '/path/to/overcloud-default-env.yaml': True,
+            '/path/to/network-isolation.json': False,
+            '/path/to/poc-custom-env.yaml': True
+        }
+
+        action = heat_capabilities.UpdateCapabilitiesAction(
+            environments, self.container_name, True)
+        self.assertEqual({
+            'environments': [
+                {'path': '/path/to/overcloud-default-env.yaml'},
+                {'path': '/path/to/poc-custom-env.yaml'}
+            ]},
+            action.run())
+        mock_cache.assert_called_once_with(
+            self.container_name,
+            "tripleo.parameters.get"
+        )
+
     @mock.patch(
         'tripleo_common.actions.base.TripleOAction.get_object_client')
     @mock.patch(
