@@ -12,60 +12,59 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import json
 import mock
 
 from heatclient import exc as heatexceptions
 from mistral.workflow import utils as mistral_workflow_utils
 from mistralclient.api import base as mistral_base
+from oslo_concurrency import processutils
 from swiftclient import exceptions as swiftexceptions
 
 from tripleo_common.actions import plan
+from tripleo_common import constants
 from tripleo_common import exception
 from tripleo_common.tests import base
 
-MAPPING_YAML_CONTENTS = """root_template: /path/to/overcloud.yaml
-root_environment: /path/to/environment.yaml
-topics:
-  - title: Fake Single Environment Group Configuration
-    description:
-    environment_groups:
-      - title:
-        description: Random fake string of text
-        environments:
-          - file: /path/to/network-isolation.json
-            title: Default Configuration
-            description:
+JSON_CONTENTS = json.dumps({
+    "environments": [{
+        "path": "overcloud-resource-registry-puppet.yaml"
+    }, {
+        "path": "environments/services/sahara.yaml"
+    }],
+    "parameter_defaults": {
+        "BlockStorageCount": 42,
+        "OvercloudControlFlavor": "yummy"
+    },
+    "passwords": {
+        "AdminPassword": "aaaa",
+        "ZaqarPassword": "zzzz"
+    },
+    "template": "overcloud.yaml",
+    "version": 1.0
+}, sort_keys=True)
 
-  - title: Fake Multiple Environment Group Configuration
-    description:
-    environment_groups:
-      - title: Random Fake 1
-        description: Random fake string of text
-        environments:
-          - file: /path/to/ceph-storage-env.yaml
-            title: Fake1
-            description: Random fake string of text
 
-      - title: Random Fake 2
-        description:
-        environments:
-          - file: /path/to/poc-custom-env.yaml
-            title: Fake2
-            description:
+YAML_CONTENTS = """
+version: 1.0
+
+template: overcloud.yaml
+environments:
+-  path: overcloud-resource-registry-puppet.yaml
+-  path: environments/services/sahara.yaml
+parameter_defaults:
+  BlockStorageCount: 42
+  OvercloudControlFlavor: yummy
+passwords:
+  AdminPassword: aaaa
+  ZaqarPassword: zzzz
 """
 
-INVALID_MAPPING_CONTENTS = """
-root_environment: /path/to/environment.yaml
-topics:
-  - title: Fake Single Environment Group Configuration
-    description:
-    environment_groups:
-      - title:
-        description: Random fake string of text
-        environments:
-          - file: /path/to/network-isolation.json
-            title: Default Configuration
-            description:
+YAML_CONTENTS_INVALID = "{bad_yaml"
+
+# `environments` is missing
+YAML_CONTENTS_MISSING_KEY = """
+template: overcloud.yaml
 """
 
 RESOURCES_YAML_CONTENTS = """heat_template_version: 2016-04-08
@@ -148,11 +147,11 @@ class CreatePlanActionTest(base.TestCase):
         super(CreatePlanActionTest, self).setUp()
         # A container that name enforces all validation rules
         self.container_name = 'Test-container-3'
-        self.capabilities_name = 'capabilities-map.yaml'
+        self.plan_environment_name = constants.PLAN_ENVIRONMENT
 
         # setup swift
         self.swift = mock.MagicMock()
-        self.swift.get_object.return_value = ({}, MAPPING_YAML_CONTENTS)
+        self.swift.get_object.return_value = ({}, YAML_CONTENTS)
         swift_patcher = mock.patch(
             'tripleo_common.actions.base.TripleOAction.get_object_client',
             return_value=self.swift)
@@ -168,47 +167,23 @@ class CreatePlanActionTest(base.TestCase):
         mistral_patcher.start()
         self.addCleanup(mistral_patcher.stop)
 
-    def test_run(self):
-
+    def test_run_success(self):
         action = plan.CreatePlanAction(self.container_name)
         action.run()
 
         self.swift.get_object.assert_called_once_with(
             self.container_name,
-            self.capabilities_name
+            self.plan_environment_name
         )
 
+        self.swift.delete_object.assert_called_once()
+
         self.mistral.environments.create.assert_called_once_with(
             name='Test-container-3',
-            variables=('{"environments":'
-                       ' [{"path": "/path/to/environment.yaml"}], '
-                       '"template": "/path/to/overcloud.yaml"}')
+            variables=JSON_CONTENTS
         )
 
-    def test_run_with_invalid_yaml(self):
-
-        self.swift.get_object.return_value = ({}, 'invalid: %')
-
-        action = plan.CreatePlanAction(self.container_name)
-        result = action.run()
-
-        error_str = 'Error parsing the yaml file'
-        # don't bother checking the exact yaml error (it's long)
-        self.assertEqual(result.error.split(':')[0], error_str)
-
-    def test_run_with_invalid_string(self):
-
-        self.swift.get_object.return_value = ({}, 'this is just a string')
-
-        action = plan.CreatePlanAction(self.container_name)
-        result = action.run()
-
-        error_str = 'Error occurred creating plan'
-        # don't bother checking the exact error (python versions different)
-        self.assertEqual(result.error.split(':')[0], error_str)
-
-    def test_run_with_invalid_plan_name(self):
-
+    def test_run_invalid_plan_name(self):
         action = plan.CreatePlanAction("invalid_underscore")
         result = action.run()
 
@@ -217,27 +192,6 @@ class CreatePlanActionTest(base.TestCase):
         # don't bother checking the exact error (python versions different)
         self.assertEqual(result.error.split(':')[0], error_str)
 
-    def test_run_with_no_file(self):
-
-        self.swift.get_object.side_effect = swiftexceptions.ClientException(
-            'atest2')
-
-        action = plan.CreatePlanAction(self.container_name)
-        result = action.run()
-
-        error_str = 'File missing from container: atest2'
-        self.assertEqual(result.error, error_str)
-
-    def test_run_with_missing_key(self):
-
-        self.swift.get_object.return_value = ({}, INVALID_MAPPING_CONTENTS)
-
-        action = plan.CreatePlanAction(self.container_name)
-        result = action.run()
-
-        error_str = "capabilities-map.yaml missing key: 'root_template'"
-        self.assertEqual(result.error, error_str)
-
     def test_run_mistral_env_already_exists(self):
         self.mistral.environments.get.side_effect = None
         self.mistral.environments.get.return_value = 'test-env'
@@ -250,6 +204,129 @@ class CreatePlanActionTest(base.TestCase):
         self.assertEqual(result.error, error_str)
         self.mistral.environments.create.assert_not_called()
 
+    def test_run_missing_file(self):
+        self.swift.get_object.side_effect = swiftexceptions.ClientException(
+            self.plan_environment_name)
+
+        action = plan.CreatePlanAction(self.container_name)
+        result = action.run()
+
+        error_str = ('File missing from container: %s' %
+                     self.plan_environment_name)
+        self.assertEqual(result.error, error_str)
+
+    def test_run_invalid_yaml(self):
+        self.swift.get_object.return_value = ({}, YAML_CONTENTS_INVALID)
+
+        action = plan.CreatePlanAction(self.container_name)
+        result = action.run()
+
+        error_str = 'Error parsing the yaml file'
+        self.assertEqual(result.error.split(':')[0], error_str)
+
+    def test_run_missing_key(self):
+        self.swift.get_object.return_value = ({}, YAML_CONTENTS_MISSING_KEY)
+
+        action = plan.CreatePlanAction(self.container_name)
+        result = action.run()
+
+        error_str = ("%s missing key: environments" %
+                     self.plan_environment_name)
+        self.assertEqual(result.error, error_str)
+
+
+class UpdatePlanActionTest(base.TestCase):
+
+    def setUp(self):
+        super(UpdatePlanActionTest, self).setUp()
+        self.container_name = 'Test-container-3'
+        self.plan_environment_name = constants.PLAN_ENVIRONMENT
+
+        # setup swift
+        self.swift = mock.MagicMock()
+        self.swift.get_object.return_value = ({}, YAML_CONTENTS)
+        swift_patcher = mock.patch(
+            'tripleo_common.actions.base.TripleOAction.get_object_client',
+            return_value=self.swift)
+        swift_patcher.start()
+        self.addCleanup(swift_patcher.stop)
+
+        # setup mistral
+        self.mistral = mock.MagicMock()
+        mistral_patcher = mock.patch(
+            'tripleo_common.actions.base.TripleOAction.get_workflow_client',
+            return_value=self.mistral)
+        mistral_patcher.start()
+        self.addCleanup(mistral_patcher.stop)
+
+    @mock.patch('tripleo_common.actions.base.TripleOAction.cache_delete')
+    def test_run_success(self, mock_cache):
+        action = plan.UpdatePlanAction(self.container_name)
+        action.run()
+
+        self.swift.get_object.assert_called_once_with(
+            self.container_name,
+            self.plan_environment_name
+        )
+
+        self.swift.delete_object.assert_called_once()
+        mock_cache.assert_called_once_with(
+            "Test-container-3",
+            "tripleo.parameters.get"
+        )
+
+        self.mistral.environments.update.assert_called_once_with(
+            name='Test-container-3',
+            variables=JSON_CONTENTS
+        )
+
+    @mock.patch('tripleo_common.actions.base.TripleOAction.cache_delete')
+    def test_run_mistral_env_missing(self, mock_cache):
+        self.mistral.environments.update.side_effect = (
+            mistral_base.APIException)
+
+        action = plan.UpdatePlanAction(self.container_name)
+        result = action.run()
+
+        error_str = ("Error updating mistral environment: %s" %
+                     self.container_name)
+        self.assertEqual(result.error, error_str)
+        self.swift.delete_object.assert_not_called()
+        mock_cache.assert_called_once_with(
+            "Test-container-3",
+            "tripleo.parameters.get"
+        )
+
+    def test_run_missing_file(self):
+        self.swift.get_object.side_effect = swiftexceptions.ClientException(
+            self.plan_environment_name)
+
+        action = plan.UpdatePlanAction(self.container_name)
+        result = action.run()
+
+        error_str = ('File missing from container: %s' %
+                     self.plan_environment_name)
+        self.assertEqual(result.error, error_str)
+
+    def test_run_invalid_yaml(self):
+        self.swift.get_object.return_value = ({}, YAML_CONTENTS_INVALID)
+
+        action = plan.UpdatePlanAction(self.container_name)
+        result = action.run()
+
+        error_str = 'Error parsing the yaml file'
+        self.assertEqual(result.error.split(':')[0], error_str)
+
+    def test_run_missing_key(self):
+        self.swift.get_object.return_value = ({}, YAML_CONTENTS_MISSING_KEY)
+
+        action = plan.UpdatePlanAction(self.container_name)
+        result = action.run()
+
+        error_str = ("%s missing key: environments" %
+                     self.plan_environment_name)
+        self.assertEqual(result.error, error_str)
+
 
 class ListPlansActionTest(base.TestCase):
 
@@ -423,3 +500,103 @@ class RoleListActionTest(base.TestCase):
         self.assertEqual(expected, result)
         self.assertEqual('overcloud.yaml', template_name)
         swift.get_object.assert_called_with(self.container, template_name)
+
+
+class ExportPlanActionTest(base.TestCase):
+
+    def setUp(self):
+        super(ExportPlanActionTest, self).setUp()
+        self.plan = 'overcloud'
+        self.delete_after = 3600
+        self.exports_container = 'plan-exports'
+
+        # setup swift
+        self.template_files = (
+            'some-name.yaml',
+            'some-other-name.yaml',
+            'yet-some-other-name.yaml',
+            'finally-another-name.yaml'
+        )
+        self.swift = mock.MagicMock()
+        self.swift.get_container.return_value = (
+            {'x-container-meta-usage-tripleo': 'plan'}, [
+                {'name': tf} for tf in self.template_files
+            ]
+        )
+        self.swift.get_object.return_value = ({}, RESOURCES_YAML_CONTENTS)
+        swift_patcher = mock.patch(
+            'tripleo_common.actions.base.TripleOAction.get_object_client',
+            return_value=self.swift)
+        swift_patcher.start()
+        self.addCleanup(swift_patcher.stop)
+
+        # setup mistral
+        self.mistral = mock.MagicMock()
+        env_item = mock.Mock()
+        env_item.variables = {
+            'template': 'overcloud.yaml',
+            'environments': [
+                {'path': 'overcloud-resource-registry-puppet.yaml'}
+            ]
+        }
+        self.mistral.environments.get.return_value = env_item
+        mistral_patcher = mock.patch(
+            'tripleo_common.actions.base.TripleOAction.get_workflow_client',
+            return_value=self.mistral)
+        mistral_patcher.start()
+        self.addCleanup(mistral_patcher.stop)
+
+    @mock.patch('tripleo_common.utils.tarball.create_tarball')
+    @mock.patch('tempfile.mkdtemp')
+    def test_run_success(self, mock_mkdtemp, mock_create_tarball):
+        get_object_mock_calls = [
+            mock.call(self.plan, tf) for tf in self.template_files
+        ]
+        get_container_mock_calls = [
+            mock.call(self.plan),
+            mock.call('plan-exports')
+        ]
+        mock_mkdtemp.return_value = '/tmp/test123'
+
+        action = plan.ExportPlanAction(self.plan, self.delete_after,
+                                       self.exports_container)
+        action.run()
+
+        self.swift.get_container.assert_has_calls(get_container_mock_calls)
+        self.swift.get_object.assert_has_calls(
+            get_object_mock_calls, any_order=True)
+        self.mistral.environments.get.assert_called_once_with(self.plan)
+        self.swift.put_object.assert_called_once()
+        mock_create_tarball.assert_called_once()
+
+    def test_run_container_does_not_exist(self):
+        self.swift.get_container.side_effect = swiftexceptions.ClientException(
+            self.plan)
+
+        action = plan.ExportPlanAction(self.plan, self.delete_after,
+                                       self.exports_container)
+        result = action.run()
+
+        error = "Error attempting an operation on container: %s" % self.plan
+        self.assertIn(error, result.error)
+
+    def test_run_environment_does_not_exist(self):
+        self.mistral.environments.get.side_effect = mistral_base.APIException
+
+        action = plan.ExportPlanAction(self.plan, self.delete_after,
+                                       self.exports_container)
+        result = action.run()
+
+        error = "The Mistral environment %s could not be found." % self.plan
+        self.assertEqual(error, result.error)
+
+    @mock.patch('tripleo_common.utils.tarball.create_tarball')
+    def test_run_error_creating_tarball(self, mock_create_tarball):
+        mock_create_tarball.side_effect = processutils.ProcessExecutionError
+
+        action = plan.ExportPlanAction(self.plan, self.delete_after,
+                                       self.exports_container)
+        result = action.run()
+
+        error = "Error while creating a tarball"
+        self.assertIn(error, result.error)
