@@ -40,6 +40,8 @@ class ScaleDownActionTest(base.TestCase):
         super(ScaleDownActionTest, self).setUp()
         self.image = collections.namedtuple('image', ['id'])
 
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'cache_delete')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_orchestration_client')
     @mock.patch('heatclient.common.template_utils.'
@@ -51,7 +53,8 @@ class ScaleDownActionTest(base.TestCase):
     @mock.patch('mistral.context.ctx')
     def test_run(self, mock_ctx, mock_get_object_client,
                  mock_get_workflow_client, mock_get_template_contents,
-                 mock_env_files, mock_get_heat_client):
+                 mock_env_files, mock_get_heat_client,
+                 mock_cache):
 
         mock_env_files.return_value = ({}, {})
         heatclient = mock.MagicMock()
@@ -112,9 +115,16 @@ class ScaleDownActionTest(base.TestCase):
             constants.STACK_TIMEOUT_DEFAULT, ['resource_id'], 'stack')
         action.run()
 
-        clear_list = list(['ComputeCount', 'ComputeRemovalPolicies'])
-        _, kwargs = heatclient.stacks.update.call_args
-        self.assertEqual(set(kwargs['clear_parameters']), set(clear_list))
-        self.assertEqual(kwargs['environment'], env)
-        self.assertEqual(kwargs['existing'], True)
-        self.assertEqual(kwargs['files'], {})
+        heatclient.stacks.update.assert_called_once_with(
+            'stack',
+            stack_name='stack',
+            template={'heat_template_version': '2016-04-30'},
+            environment=env,
+            existing=True,
+            files={},
+            timeout_mins=240)
+
+        mock_cache.assert_called_once_with(
+            "stack",
+            "tripleo.parameters.get"
+        )
